Package: feature
Version: 1.2.16
Date: 2026-05-11
Title: Local Inferential Feature Significance for Multivariate Kernel
        Density Estimation
Authors@R: c(person("Tarn", "Duong", role=c("aut","cre"), email="tarn.duong@gmail.com", comment=c(ORCID="0000-0002-1198-3482")),
             person("Matt", "Wand", role="ctb", email="Matt.Wand@uts.edu.au", comment=c(ORCID="0000-0003-2555-896X")))
Maintainer: Tarn Duong <tarn.duong@gmail.com>
Depends: R (>= 1.4.0)
Imports: ks (>= 1.12.0), plot3D, tcltk
Suggests: knitr, misc3d, rgl, rmarkdown, MASS
VignetteBuilder: knitr
Description: Local inferential feature significance for multivariate kernel density estimation.
License: GPL-2 | GPL-3
URL: https://mvstat.net/feature/, https://github.com/cran/feature/
NeedsCompilation: no
Packaged: 2026-05-11 13:25:42 UTC; tduong
Author: Tarn Duong [aut, cre] (ORCID: <https://orcid.org/0000-0002-1198-3482>),
  Matt Wand [ctb] (ORCID: <https://orcid.org/0000-0003-2555-896X>)
Repository: CRAN
Date/Publication: 2026-05-11 16:50:06 UTC
Built: R 4.5.2; ; 2026-05-11 20:15:50 UTC; unix
