## Introduction 

The `feature` package contains functions to display and compute kernel density estimates, significant gradient and significant curvature regions. Significant gradient and/or curvature regions often correspond to significant features (e.g. local modes). 

There are two main functions in this package. `featureSignifGUI()` is the interactive function where the user can select bandwidths from a pre-defined range. This mode is useful for initial exploratory data analysis. `featureSignif()` is the non-interactive function. This is useful when the user has a more definite idea of suitable values for the bandwidths. The latter is closely related to the `ks::kfs()` function. 

For a more detailed example for 1-, 2- and 3-d data, see `vignette("feature")`. 

## Installation

Install from CRAN:

```r
install.packages("feature")
```

## Further reading

Duong, T., Cowling, A., Koch, I., and Wand, M. P. (2008) [Feature significance for multivariate kernel density estimation](http://mvstat.net/tduong/research/publications/duong-et-al-2008-csda.pdf) _Computational Statistics and Data Analysis_ **52**, 4225--4242.